﻿using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using static System.Windows.Forms.AxHost;
using Timer = System.Windows.Forms.Timer;

namespace Kivii.Client.Invoices.Etax
{
    public partial class FrmMain : Form
    {
        private JsonServiceClient _client = null;

        private Timer _timer = null;

        public FrmMain()
        {
            InitializeComponent();
        }

        #region 界面控件默认值管理
        private void loadDefault(params Control[] ctls)
        {
            using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
            {
                foreach (var ctl in ctls)
                {
                    var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                    if (setting == null) continue;
                    ctl.Text = setting.ControlValue;
                }
            }
        }

        private void setDefault(params Control[] ctls)
        {
            using (var conn = KiviiContext.GetOpenedDbConnection<UiSetting>())
            {
                foreach (var ctl in ctls)
                {
                    var setting = conn.Select<UiSetting>(o => o.FormName == this.Name && o.ControlName == ctl.Name).FirstOrDefault();
                    if (setting == null)
                    {
                        setting = new UiSetting();
                        setting.FormName = this.Name;
                        setting.ControlName = ctl.Name;
                        setting.ControlValue = ctl.Text;
                        conn.Insert(setting);
                    }
                    else
                    {
                        setting.ControlValue = ctl.Text;
                        conn.Update(setting);
                    }
                }
            }
        }

        /// <summary>
        /// 输出消息
        /// </summary>
        /// <param name="message">消息内容</param>
        /// <param name="clearMessage">是否清除消息</param>
        private void outputMessage(string message, bool clearMessage = false)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new Action(() => outputMessage(message, clearMessage)));
                return;
            }
            if (clearMessage) rtbMessage.Clear();
            if (rtbMessage.Lines.Count() > 500) rtbMessage.Clear();
            var time = DateTime.Now.ToString("HH:mm:ss");
            rtbMessage.AppendText($"[{time}]{message}{Environment.NewLine}");
            rtbMessage.ScrollToCaret();
        }

        #endregion

        private void FrmMain_Load(object sender, EventArgs e)
        {
            btnStartTimer.Enabled = true;
            btnStopTimer.Enabled = !btnStartTimer.Enabled;
            loadDefault(tbxServiceUrl, tbxUserName, tbxPassword, tbFilePath);
        }

        private void btnLogin_Click(object sender, EventArgs e)
        {
            if (_client != null)
            {
                MessageBox.Show("登陆前先退出原连接");
                return;
            }
            if (tbFilePath.Text.IsNullOrEmpty())
            {
                MessageBox.Show("请先设置读取路径！");
                return;
            }
            btnLogin.Enabled = false;
            _client = new JsonServiceClient(tbxServiceUrl.Text);
            var auth = new Authenticate { UserName = tbxUserName.Text, Password = tbxPassword.Text, provider = "Kivii" };
            var task = _client.PostAsync(auth);
            task.Success(resp =>
            {
                btnLogin.Enabled = resp.Kvid == Guid.Empty ? true : false;
                btnLogout.Enabled = !btnLogin.Enabled;
                tbxServiceUrl.Enabled = btnLogin.Enabled;
                tbxUserName.Enabled = btnLogin.Enabled;
                tbxPassword.Enabled = btnLogin.Enabled;
                btnSelectFileFolder.Enabled = btnLogin.Enabled;
                setDefault(tbxServiceUrl, tbxUserName, tbxPassword, tbFilePath);
                outputMessage($"登录成功：{resp.FullName}");
                Thread.Sleep(100);
                timerStart();
            }, true);
            task.Error(err =>
            {
                btnLogin.Enabled = true;
                btnLogout.Enabled = !btnLogin.Enabled;
                tbxServiceUrl.Enabled = btnLogin.Enabled;
                tbxUserName.Enabled = btnLogin.Enabled;
                tbxPassword.Enabled = btnLogin.Enabled;
                btnSelectFileFolder.Enabled = btnLogin.Enabled;
                _client.Dispose();
                _client = null;
                MessageBox.Show(err.Message, "Login Error");
            }, true);
        }

        private void btnLogout_Click(object sender, EventArgs e)
        {
            if (_client == null)
            {
                MessageBox.Show("未连接");
                return;
            }
            btnLogout.Enabled = false;
            var auth = new Authenticate { provider = "Logout" };
            var task = _client.PostAsync(auth);
            task.Success(resp =>
            {
                btnLogin.Enabled = resp != null;
                btnLogout.Enabled = !btnLogin.Enabled;
                tbxServiceUrl.Enabled = btnLogin.Enabled;
                tbxUserName.Enabled = btnLogin.Enabled;
                tbxPassword.Enabled = btnLogin.Enabled;
                btnSelectFileFolder.Enabled = btnLogin.Enabled;
                outputMessage($"退出成功");
                _client.Dispose();
                _client = null;
                Thread.Sleep(100);
                timerStop();
            });
            task.Error(err =>
            {
                btnLogout.Enabled = true;
                MessageBox.Show(err.Message, "Logout Error");
            });
        }

        private void btnSelectFileFolder_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog fbd = new FolderBrowserDialog();
            if (fbd.ShowDialog() != DialogResult.OK) return;
            tbFilePath.Text = fbd.SelectedPath;
            setDefault(tbFilePath);
        }


        private void timerStart()
        {
            if (_client == null)
            {
                MessageBox.Show("请先登录！");
                return;
            }
            if (tbFilePath.Text.IsNullOrEmpty())
            {
                MessageBox.Show("请先设置读取路径！");
                return;
            }
            if (_timer == null)
            {
                _timer = new Timer();
                _timer.Interval = 3 * 1000;
                _timer.Tick += _timer_Tick;
            }
            _timer.Start();
            btnStartTimer.Enabled = false;
            btnStopTimer.Enabled = true;
            outputMessage("定时器已启动");
        }
        private void timerStop()
        {
            if (_timer != null) _timer.Stop();
            _timer = null;
            btnStartTimer.Enabled = true;
            btnStopTimer.Enabled = false;
            outputMessage("定时器已停止");
        }

        private void _timer_Tick(object sender, EventArgs e)
        {
            try
            {
                _timer.Stop();
                readFilePath();
            }
            catch (Exception ex)
            {
                outputMessage($"定时器错误：{ex.Message}");
            }
            finally
            {
                if (_timer != null) _timer.Start();
            }
        }

        private void readFilePath()
        {
            if (InvokeRequired)
            {
                this.Invoke(new Action(readFilePath));
                return;
            }
            #region 文件夹路径设置
            if (tbFilePath.Text.IsNullOrEmpty())
            {
                outputMessage("未设置文件读取路径");
                timerStop();
                return;
            }
            var path = tbFilePath.Text;
            var uploadedPath = $"{path}\\uploaded";
            var errorPath = $"{path}\\error";
            if (!Directory.Exists(path))
            {
                Directory.CreateDirectory(path);
                outputMessage($"自动设置路径：{path}");
            }
            if (!Directory.Exists(uploadedPath))
            {
                Directory.CreateDirectory(uploadedPath);
                outputMessage($"自动设置路径：{uploadedPath}");
            }
            if (!Directory.Exists(errorPath))
            {
                Directory.CreateDirectory(errorPath);
                outputMessage($"自动设置路径：{errorPath}");
            }
            #endregion

            var pdfPaths = Directory.GetFiles(path, "*.pdf");
            //if (pdfPaths.Length > 0) outputMessage($"找到pdf文件：{pdfPaths.Length}个");
            var ofdPaths = Directory.GetFiles(path, "*.ofd");
            //if (ofdPaths.Length > 0) outputMessage($"找到ofd文件：{ofdPaths.Length}个");
            List<string> filePaths = new List<string>();
            filePaths.AddRange(pdfPaths);
            filePaths.AddRange(ofdPaths);
            if (filePaths.Count > 0) outputMessage($"找到文件：{filePaths.Count}个");
            if (!filePaths.IsNullOrEmpty())
            {
                foreach (var filePath in filePaths)
                {
                    var file = new DirectoryInfo(filePath);
                    var split = file.Name.Split('_');
                    if (split.Length < 2)
                    {
                        var mvPath = $"{errorPath}\\{file.Name}";
                        fileMove(file.FullName, mvPath);
                        outputMessage($"文件名：{file.Name}格式不正确，无法从中获取发票号，此文件已移动到目录：{errorPath}");
                    }
                    var serialNumber = split[1];
                    var request = new RequestInvoice();
                    request.OffsetKvid = Guid.Empty;
                    request.ParentKvid = Guid.Empty;
                    request.SerialNumber = serialNumber;
                    var resp = _client.Post(request);
                    if (resp != null)
                    {
                        if (resp.Results.IsNullOrEmpty())
                        {
                            var mvPath = $"{errorPath}\\{file.Name}";
                            fileMove(file.FullName, mvPath);
                            outputMessage($"文件名：{file.Name}格式不正确，无法从中获取发票号，此文件已移动到目录：{errorPath}");
                        }
                        else
                        {

                        }
                    }
                }
            }

        }

        private void btnStartTimer_Click(object sender, EventArgs e)
        {
            timerStart();
        }

        private void btnStopTimer_Click(object sender, EventArgs e)
        {
            timerStop();
        }

        private void fileMove(string sourcePath, string destinationPath)
        {
            if (File.Exists(destinationPath))
            {
                File.Delete(destinationPath);
            }
            if (File.Exists(sourcePath))
            {
                File.Move(sourcePath, destinationPath);
            }
        }
    }
}
